window.Gleam.initSearch([{"doc":"cat","title":"cat","content":"# Category Theory\n\n[![Package Version](https://img.shields.io/hexpm/v/cat)](https://hex.pm/packages/cat)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/cat)\n\nThis package implements several category theory concepts, following [this book](https://bartoszmilewski.com/2014/10/28/category-theory-for-programmers-the-preface/) by Bartosz Milewski.\n\n```sh\ngleam add cat\n```\n\nFunctor Example\n\n```gleam\nimport cat/functor as fun\nimport cat/instances/functor as inst\nimport gleam/option\nimport gleam/io\n\npub fn main() {\n\n  option.Some([1, 2, 3])\n  |&gt; fun.functor_compose(inst.list_functor(), inst.option_functor())(fn(x) {\n    x % 2 == 0\n  })\n  |&gt; io.debug()\n  // -&gt; option.Some([True, False, True])\n}\n```\n\nApplicative Example\n\n```gleam\nimport cat/applicative as app\nimport cat/instances/applicative as inst\nimport cat/functor as fun\nimport gleam/option.{None, Some}\nimport gleam/io\n\npub fn main() {\n  [Some(1), None, Some(3)]\n  |&gt; {\n    [fn(x) { x * 2 }, fn(x) { x + 10 }]\n    |&gt; fun.list_functor().fmap(fun.option_functor().fmap)\n    |&gt; app.apply(inst.list_applicative())\n  }\n  |&gt; io.debug()\n  // -&gt; [Some(2), None, Some(6), Some(11), None, Some(13)]\n}\n```\n\nMonoid Example\n\n```gleam\nimport cat.{type Either, Left, Right}\nimport cat/monoid as mono\nimport gleam/io\n\npub fn main() {\n   let either_sum_monoid =\n    mono.Monoid(\n      mempty: Left(0),\n      mappend: fn(e1: Either(Int, String), e2: Either(Int, String)) -&gt; Either(Int, String) {\n        case e1, e2 {\n          Right(s), _ -&gt; Right(s)\n          _, Right(s) -&gt; Right(s)\n          Left(a), Left(b) -&gt; Left(a + b)\n        }\n      }\n    )\n\n  either_sum_monoid\n  |&gt; mono.mconcat([Left(2), Left(3), Left(4)])\n  |&gt; io.debug()\n  // -&gt; Left(9)\n\n  either_sum_monoid\n  |&gt; mono.mconcat([Left(2), Right(&quot;error&quot;), Left(4)])\n  |&gt; io.debug()\n  // -&gt; Right(&quot;error&quot;)\n}\n```\n\nBifunctor Example\n\n```gleam\nimport cat\nimport cat/bifunctor as bf\nimport cat/instances/functor as fun\nimport cat/instances/bifunctor as inst\nimport gleam/io\n\npub fn main() {\n  // Either bifunctor\n  let either_bf = inst.either_bifunctor()\n  // Const () functor\n  let const_f = fun.const_functor()\n  // Identity functor\n  let id_f = fun.identity_functor()\n\n  // Constructing the maybe functor:\n  // Maybe b = Either (Const () a) (Identity b)\n  let maybe_functor = fn() -&gt; bf.Bifunctor(\n    bf.BiCompF(bf.EitherBF, fun.ConstF(Nil), fun.IdentityF),\n    a, b, c, d,\n    cat.Either(cat.Const(Nil, a), cat.Identity(b)),\n    cat.Either(cat.Const(Nil, c), cat.Identity(d)),\n  ) {\n    bf.bifunctor_compose(either_bf, const_f, id_f)\n  }\n\n  cat.Left(cat.Const(Nil))\n  |&gt; maybe_functor().bimap(fn(_) { panic }, int.to_string)\n  |&gt; io.debug()\n  // -&gt; cat.Left(cat.Const(Nil))\n\n  cat.Right(cat.Identity(3))\n  |&gt; maybe_functor().bimap(fn(_) { panic }, int.to_string)\n  |&gt; io.debug()\n  // -&gt; cat.Right(cat.Identity(&quot;3&quot;))\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/cat&gt;.\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\n```\n","url":"index.html"},{"doc":"cat","title":"Const","content":"pub type Const(c, a) {\n  Const(c)\n}\n `Const` type from Haskell.\n \n data Const c a = Const c\n \n Only the first parameter affects the type, the second is ignore. \\\n In gleam, we say that `a` is a `phantom type`.\n ### Examples\n \n let x: Const(Int, Bool) = Const(7)\n let y: Const(String, String) = Const(&quot;abc&quot;)\n \nConst(c)\n\n\nSynonyms:\ncat.Const\ncat Const","url":"cat.html#Const"},{"doc":"cat","title":"Either","content":"pub type Either(a, b) {\n  Left(a)\n  Right(b)\n}\n Canonical implementation of a `coproduct` (sum type).\n \n data Either a b = Left a | Right b\n \n ### Examples\n \n let check_positive = fn(x: Int) -&gt; Either(Int, String) {\n   case x &gt;= 0 {\n     True -&gt; Left(x)\n     False -&gt; Right(&quot;negative number&quot;)\n   }\n }\n check_positive(12)\n // -&gt; Left(12)\n check_positive(-3)\n // -&gt; Right(&quot;negative number&quot;)\n \nLeft(a)\n\n\nRight(b)\n\n\nSynonyms:\ncat.Either\ncat Either","url":"cat.html#Either"},{"doc":"cat","title":"Identity","content":"pub type Identity(a) {\n  Identity(a)\n}\n `Identity` type from Haskell\n \n newtype Identity a = Identity a\n \n ### Examples\n \n Identity(8)\n Identity(&quot;abc&quot;)\n \nIdentity(a)\n\n\nSynonyms:\ncat.Identity\ncat Identity","url":"cat.html#Identity"},{"doc":"cat","title":"List","content":"pub type List(a) {\n  Null\n  Cons(List(a))\n}\n\nNull\n\n\nCons(List(a))\n\n\nSynonyms:\ncat.List\ncat List","url":"cat.html#List"},{"doc":"cat","title":"Maybe","content":"pub type Maybe(a) {\n  Nothing\n  Just(a)\n}\n `Maybe` type from Haskell (`Option` in gleam).\n \n data Maybe = Nothing | Just a\n // Equivalent: Sum type between `unit` and `a`\n type Maybe = Either () a\n \n ### Examples\n \n let safe_div = fn(a, b) {\n   case b != 0.0 {\n     True -&gt; Just(a /. b)\n     False -&gt; Nothing\n   }\n }\n safe_div(3.0, 4.0)\n // -&gt; Just(0,75)\n safe_div(3.0, 0.0)\n // -&gt; Nothing\n \nNothing\n\n\nJust(a)\n\n\nSynonyms:\ncat.Maybe\ncat Maybe","url":"cat.html#Maybe"},{"doc":"cat","title":"Pair","content":"pub type Pair(a, b) {\n  Pair(fst: a, snd: b)\n}\n Canonical implementation of a `product` (tuple).\n Examples\n \n let even_to_string = fn(x: Int) -&gt; Pair(String, Bool) {\n   Pair(int.to_string(x), x % 2 == 0)\n }\n even_to_string(82).fst\n // -&gt; &quot;82&quot;\n even_to_string(82).snd\n // -&gt; True\n even_to_string(83).snd\n // -&gt; False\n \nPair(fst: a, snd: b)\n\n\nSynonyms:\ncat.Pair\ncat Pair","url":"cat.html#Pair"},{"doc":"cat","title":"Reader","content":"pub type Reader(r, a) {\n  Reader(apply: fn(r) -&gt; a)\n}\n Encapsulates a function.\n \n type Reader r a = r -&gt; a\n \n ### Examples\n \n let r = Reader(fn(x) { x % 2 == 1 })\n r.apply(6)\n // -&gt; False\n \nReader(apply: fn(r) -&gt; a)\n\n\nSynonyms:\ncat.Reader\ncat Reader","url":"cat.html#Reader"},{"doc":"cat","title":"Void","content":"pub type Void {\n  Void(Void)\n}\n A type corresponding to an `empty set`. It is `not inhabited` by any values.\nVoid(Void)\n\n\nSynonyms:\ncat.Void\ncat Void","url":"cat.html#Void"},{"doc":"cat","title":"Writer","content":"pub type Writer(a) {\n  Writer(a, String)\n}\n Encapsulates a pair whose first component is a `value` of arbitrary type a and the second component is a `string`. \\\n Used to `embellish` the return values of functions.\n ### Examples\n \n // Original function\n f = fn(x) {x * 2}\n // Embellished function\n f = fn(x) {Writer(x * 2, &quot;doubled &quot;)}\n \nWriter(a, String)\n\n\nSynonyms:\ncat.Writer\ncat Writer","url":"cat.html#Writer"},{"doc":"cat","title":"absurd","content":"pub fn absurd(: Void) -&gt; a\n A function that can `never` be called. It is polymorphic in the return type. \nSynonyms:\ncat.absurd\ncat absurd","url":"cat.html#absurd"},{"doc":"cat","title":"compose","content":"pub fn compose(g: fn(a) -&gt; b, f: fn(c) -&gt; a) -&gt; fn(c) -&gt; b\n Given a function `f` that takes an argument of type A and returns a B, and another function `g` that takes a B and returns a C, you can `compose` them by `passing the result of f to g`. \n \n (∘) :: (b -&gt; c) -&gt; (a -&gt; b) -&gt; (a -&gt; c)\n (g ∘ f) x = g (f x)\n \n Properties of composition:\n - Associativity h ∘ (g ∘ f) == (h ∘ g) ∘ f == h ∘ g ∘ f\n - Identity see [`id`](#id) for more info\n ### Examples\n \n let f = fn(x: Int) { int.to_string(x) }\n let g = fn(s: String) { s == &quot;28&quot; }\n let h = compose(g, f)\n // -&gt; h takes an int, transforms it into a string, then compares it to &quot;28&quot; and returns a bool\n  \nSynonyms:\ncat.compose\ncat compose","url":"cat.html#compose"},{"doc":"cat","title":"constant","content":"pub fn constant(x: a) -&gt; fn(b) -&gt; a\n `const` function.\n ### Examples\n \n constant(1)(2)\n // -&gt; 1\n \nSynonyms:\ncat.constant\ncat constant","url":"cat.html#constant"},{"doc":"cat","title":"coproduct_factorizer","content":"pub fn coproduct_factorizer(\n  i: fn(a) -&gt; b,\n  j: fn(c) -&gt; b,\n) -&gt; fn(Either(a, c)) -&gt; b\n Produces the factorizing function from a `candidate` c with `two injections` i and j to the best coproduct (either). \\\n Property: i and j can be `reconstructed` from the canonical coproduct e  \\\n With m = coproduct_factorizer(i, j), we have:\n - i(x) = m(Left(x))\n - j(x) = m(Right(x))\n ### Examples\n \n // Given the candidate #(Int, Bool) with two injections from Int and Bool\n let i = fn(x: Int) {#(x, True)}\n let j = fn(x: Bool) {#(9, x)}\n // We show that Either(Int, Bool) is a better coproduct by finding the mapping m:\n let m = coproduct_factorizer(i, j)\n m(Left(2))\n // -&gt; #(2, True)  \n m(Right(False))\n // -&gt; #(9, False)  \n \nSynonyms:\ncat.coproduct_factorizer\ncat coproduct_factorizer","url":"cat.html#coproduct_factorizer"},{"doc":"cat","title":"curry","content":"pub fn curry(f: fn(a, b) -&gt; c) -&gt; fn(a) -&gt; fn(b) -&gt; c\n Transforms a function that takes `2 arguments` into a function that takes `one argument` and returns a `partial function`.\n ### Examples\n \n let map_123 = [1, 2, 3] |&gt; curry(list.map)\n map_123(fn(x) { x + 1 })\n // -&gt; [2, 3, 4]\n map_123(fn(x) { x * 2 })\n // -&gt; [2, 4, 6]\n \nSynonyms:\ncat.curry\ncat curry","url":"cat.html#curry"},{"doc":"cat","title":"fish","content":"pub fn fish(\n  m1: fn(a) -&gt; Writer(b),\n  m2: fn(b) -&gt; Writer(c),\n) -&gt; fn(a) -&gt; Writer(c)\n `Composition` for the embellished functions that return the Writer type.\n \n (&gt;=&gt;) :: (a -&gt; Writer b) -&gt; (b -&gt; Writer c) -&gt; (a -&gt; Writer c)\n m1 &gt;=&gt; m2 = \\x -&gt; \n   let (y, s1) = m1 x\n       (z, s2) = m2 y\n   in (z, s1 ++ s2)\n \n ### Examples\n \n let up_case = fn(s: String) { Writer(string.uppercase(s), &quot;upCase &quot;) }\n let to_words = fn(s: String) { Writer(string.split(s, &quot; &quot;), &quot;toWords &quot;) }\n let process = fish(up_case, to_words)\n process(&quot;Anna has apples&quot;)\n // -&gt; Writer([&quot;ANNA&quot;, &quot;HAS&quot;, &quot;APPLES&quot;], &quot;upCase toWords &quot;)\n \nSynonyms:\ncat.fish\ncat fish","url":"cat.html#fish"},{"doc":"cat","title":"flip","content":"pub fn flip(f: fn(a, b) -&gt; c) -&gt; fn(b, a) -&gt; c\n `flip` function.\n ### Examples\n \n let f = fn(x: Int, y: Bool) { int.to_string(x) &lt;&gt; &quot; &quot; &lt;&gt; bool.to_string(y) }\n flip(f)(True, 23)\n // -&gt; &quot;23 True&quot;\n \nSynonyms:\ncat.flip\ncat flip","url":"cat.html#flip"},{"doc":"cat","title":"id","content":"pub fn id(x: a) -&gt; a\n The `identity function` is a `unit of composition`.\n \n id :: a -&gt; a\n id a = a\n \n It follows the identity conditions:\n - f ∘ id == f\n - id ∘ f == f\n ### Examples\n \n id(3)\n // -&gt; 3\n id(&quot;abc&quot;)\n // -&gt; &quot;abc&quot;\n \nSynonyms:\ncat.id\ncat id","url":"cat.html#id"},{"doc":"cat","title":"main","content":"pub fn main() -&gt; Nil\n\nSynonyms:\ncat.main\ncat main","url":"cat.html#main"},{"doc":"cat","title":"maybe_compose","content":"pub fn maybe_compose(\n  m1: fn(a) -&gt; Maybe(b),\n  m2: fn(b) -&gt; Maybe(c),\n) -&gt; fn(a) -&gt; Maybe(c)\n `Composition` for the Maybe type \n ### Examples\n \n let safe_reciprocal = fn(x) {\n   case x != 0.0 {\n     True -&gt; Just(1.0 /. x)\n     False -&gt; Nothing\n   }\n }\n let safe_root = fn(x) {\n   case x &gt;=. 0.0 {\n     True -&gt; Just(x |&gt; float.square_root() |&gt; result.unwrap(0.0))\n     False -&gt; Nothing\n   }\n }\n let safe_reciprocal_root = maybe_compose(safe_reciprocal, safe_root)\n // -&gt; a function that calculates sqrt(1/x)\n safe_reciprocal_root(0.25)\n // -&gt; Just(2.0)\n safe_reciprocal_root(0.0)\n // -&gt; Nothing\n safe_reciprocal_root(-2.0)\n // -&gt; Nothing\n \nSynonyms:\ncat.maybe_compose\ncat maybe_compose","url":"cat.html#maybe_compose"},{"doc":"cat","title":"maybe_id","content":"pub fn maybe_id(x: a) -&gt; Maybe(a)\n The `idenitity morphism` for the Maybe type.\n ### Examples\n \n maybe_id(25)\n // -&gt; Just(25)\n maybe_id(Nothing)\n // -&gt; Just(Nothing)\n \nSynonyms:\ncat.maybe_id\ncat maybe_id","url":"cat.html#maybe_id"},{"doc":"cat","title":"maybe_to_option","content":"pub fn maybe_to_option(m: Maybe(a)) -&gt; Option(a)\n Converts from `Maybe` to gleam `Option`.\n ### Examples\n \n maybe_to_option(Nothing)\n // -&gt; None\n maybe_to_option(Just(2))\n // -&gt; Some(2)\n \nSynonyms:\ncat.maybe_to_option\ncat maybe_to_option","url":"cat.html#maybe_to_option"},{"doc":"cat","title":"option_to_maybe","content":"pub fn option_to_maybe(o: Option(a)) -&gt; Maybe(a)\n Converts from gleam `Option` to `Maybe`.\n ### Examples\n \n option_to_maybe(None)\n // -&gt; Nothing\n option_to_maybe(Some(2))\n // -&gt; Just(2)\n \nSynonyms:\ncat.option_to_maybe\ncat option_to_maybe","url":"cat.html#option_to_maybe"},{"doc":"cat","title":"pair_to_tuple","content":"pub fn pair_to_tuple(p: Pair(a, b)) -&gt; #(a, b)\n Converts from `Pair` to gleam `Tuple`.\n ### Examples\n \n pair_to_tuple(Pair(2, True))\n // -&gt; #(2, True)\n \nSynonyms:\ncat.pair_to_tuple\ncat pair_to_tuple","url":"cat.html#pair_to_tuple"},{"doc":"cat","title":"product_factorizer","content":"pub fn product_factorizer(\n  p: fn(a) -&gt; b,\n  q: fn(a) -&gt; c,\n) -&gt; fn(a) -&gt; Pair(b, c)\n Produces the factorizing function from a `candidate` c with `two projections` p and q to the best product (tuple / pair). \\\n Property: p and q can be `reconstructed` from the canonical product  \\\n With m = product_factorizer(p, q), we have:\n - p(x) = m(x).fst\n - q(x) = m(x).snd\n ### Examples\n \n // Given the candidate Int with two projections to Int and Bool\n let p = fn(x: Int) {x}\n let q = fn(_: Int) {True}\n // We show that Pair(Int, Bool) is a better product by finding the mapping m:\n let m = product_factorizer(p, q)\n m(7)\n // -&gt; Pair(7, True)  \n \nSynonyms:\ncat.product_factorizer\ncat product_factorizer","url":"cat.html#product_factorizer"},{"doc":"cat","title":"tuple_to_pair","content":"pub fn tuple_to_pair(t: #(a, b)) -&gt; Pair(a, b)\n Converts from gleam `Tuple` to `Pair`.\n ### Examples\n \n tuple_to_pair(#(2, True))\n // -&gt; Pair(2, True)\n \nSynonyms:\ncat.tuple_to_pair\ncat tuple_to_pair","url":"cat.html#tuple_to_pair"},{"doc":"cat","title":"uncurry","content":"pub fn uncurry(g: fn(a) -&gt; fn(b) -&gt; c) -&gt; fn(a, b) -&gt; c\n Transforms a function that takes `one argument` and returns a `partial function` into a function that takes `2 arguments.\n ### Examples\n \n let add_partial = fn(x) { fn(y) { x + y } }\n let add = uncurry(add_partial)\n add(2, 5)\n // -&gt; 7\n add(1, 1)\n // -&gt; 2\n \nSynonyms:\ncat.uncurry\ncat uncurry","url":"cat.html#uncurry"},{"doc":"cat","title":"unit","content":"pub fn unit(: a) -&gt; Nil\n A function from any type to a unit (Nil in gleam).\n \n unit :: a -&gt; ()\n unit _ = ()\n \n ### Examples\n \n unit(42)\n // -&gt; Nil\n unit(True)\n // -&gt; Nil\n \nSynonyms:\ncat.unit\ncat unit","url":"cat.html#unit"},{"doc":"cat","title":"cat","content":" `Basic category concepts` \\\n Functions: composition, identity, unit, constant, flip, curry, uncurry. \\\n Types: Void, option (Maybe), product (Pair), coproduct (Either), Identity, and Const.","url":"cat.html"},{"doc":"cat/algebra","title":"alpha","content":"pub fn alpha(p: Pair(a, Pair(b, c))) -&gt; Pair(Pair(a, b), c)\n Associativity `up to isomorphism` of nested Pairs. \\\n __a * (b * c) = (a * b) * c__\nSynonyms:\ncat/algebra.alpha\ncat/algebra alpha","url":"cat/algebra.html#alpha"},{"doc":"cat/algebra","title":"alpha_inv","content":"pub fn alpha_inv(p: Pair(Pair(a, b), c)) -&gt; Pair(a, Pair(b, c))\n Inverse associativity `up to isomorphism` of nested Pairs. \\\n __(a * b) * c = a * (b * c)__\nSynonyms:\ncat/algebra.alpha_inv\ncat/algebra alpha_inv","url":"cat/algebra.html#alpha_inv"},{"doc":"cat/algebra","title":"beta","content":"pub fn beta(\n  e1: Either(a, Either(b, c)),\n) -&gt; Either(Either(a, b), c)\n Associativity `up to isomorphism` of nested Eithers. \\\n __a + (b + c) = (a + b) + c__\nSynonyms:\ncat/algebra.beta\ncat/algebra beta","url":"cat/algebra.html#beta"},{"doc":"cat/algebra","title":"beta_inv","content":"pub fn beta_inv(\n  e1: Either(Either(a, b), c),\n) -&gt; Either(a, Either(b, c))\n Inverse associativity `up to isomorphism` of nested Eithers. \\\n __(a + b) + c = a + (b + c)__\nSynonyms:\ncat/algebra.beta_inv\ncat/algebra beta_inv","url":"cat/algebra.html#beta_inv"},{"doc":"cat/algebra","title":"composition","content":"pub fn composition(\n  p: Pair(fn(a) -&gt; b, fn(b) -&gt; Void),\n) -&gt; fn(a) -&gt; Void\n `Modus Tollens`. \\\n __(a -&gt; b) ∧ ¬b -&gt; ¬a__\nSynonyms:\ncat/algebra.composition\ncat/algebra composition","url":"cat/algebra.html#composition"},{"doc":"cat/algebra","title":"delta","content":"pub fn delta(e: Either(a, a)) -&gt; Pair(Bool, a)\n Morphism from `Either(a, a)` to `Pair(Bool, a)`. \\\n __a + a = 2 * a__\nSynonyms:\ncat/algebra.delta\ncat/algebra delta","url":"cat/algebra.html#delta"},{"doc":"cat/algebra","title":"delta_inv","content":"pub fn delta_inv(p: Pair(Bool, a)) -&gt; Either(a, a)\n Morphism from `Pair(Bool, a)` to `Either(a, a)`. \\\n __2 * a = a + a__\nSynonyms:\ncat/algebra.delta_inv\ncat/algebra delta_inv","url":"cat/algebra.html#delta_inv"},{"doc":"cat/algebra","title":"epsilon","content":"pub fn epsilon(\n  f: fn(Either(a, b)) -&gt; c,\n) -&gt; Pair(fn(a) -&gt; c, fn(b) -&gt; c)\n `Exponentials of Sums`. \\\n __a ^ (b + c) = (a ^ b) * (a ^ c)__\nSynonyms:\ncat/algebra.epsilon\ncat/algebra epsilon","url":"cat/algebra.html#epsilon"},{"doc":"cat/algebra","title":"epsilon_inv","content":"pub fn epsilon_inv(\n  p: Pair(fn(a) -&gt; b, fn(c) -&gt; b),\n) -&gt; fn(Either(a, c)) -&gt; b\n Inverse `Exponentials of Sums`. \\\n __(a ^ b) * (a ^ c) = a ^ (b + c)__\nSynonyms:\ncat/algebra.epsilon_inv\ncat/algebra epsilon_inv","url":"cat/algebra.html#epsilon_inv"},{"doc":"cat/algebra","title":"eta","content":"pub fn eta(unit_fun: fn(a) -&gt; Nil) -&gt; Nil\n `Powers of One`. \\\n __1 ^ a = 1__\nSynonyms:\ncat/algebra.eta\ncat/algebra eta","url":"cat/algebra.html#eta"},{"doc":"cat/algebra","title":"eta_inv","content":"pub fn eta_inv(unit: Nil) -&gt; fn(a) -&gt; Nil\n Inverse `Powers of One`. \\\n __1 = 1 ^ a__\nSynonyms:\ncat/algebra.eta_inv\ncat/algebra eta_inv","url":"cat/algebra.html#eta_inv"},{"doc":"cat/algebra","title":"eval","content":"pub fn eval(p: Pair(fn(a) -&gt; b, a)) -&gt; b\n `Modus Ponens`. \\\n __((a -&gt; b) ∧ a) -&gt; b__\nSynonyms:\ncat/algebra.eval\ncat/algebra eval","url":"cat/algebra.html#eval"},{"doc":"cat/algebra","title":"gamma","content":"pub fn gamma(m: Maybe(a)) -&gt; Either(Const(Nil, b), Identity(a))\n Morphism from `Maybe b` to `Either (Const () a) (Identity b)`.\nSynonyms:\ncat/algebra.gamma\ncat/algebra gamma","url":"cat/algebra.html#gamma"},{"doc":"cat/algebra","title":"gamma_inv","content":"pub fn gamma_inv(\n  e: Either(Const(Nil, a), Identity(b)),\n) -&gt; Maybe(b)\n Inverse morphism from `Either (Const () a) (Idenity b)` to `Maybe b`.\nSynonyms:\ncat/algebra.gamma_inv\ncat/algebra gamma_inv","url":"cat/algebra.html#gamma_inv"},{"doc":"cat/algebra","title":"not","content":"pub fn not(: a) -&gt; fn(a) -&gt; Void\n `Logical not`. \\\n __¬p = p -&gt; False__\nSynonyms:\ncat/algebra.not\ncat/algebra not","url":"cat/algebra.html#not"},{"doc":"cat/algebra","title":"omega","content":"pub fn omega(m: Maybe(a)) -&gt; Either(Nil, a)\n Morphism from `Maybe(a)` to `Either(Nil, a)`. \\\n __1 + a__\nSynonyms:\ncat/algebra.omega\ncat/algebra omega","url":"cat/algebra.html#omega"},{"doc":"cat/algebra","title":"omega_inv","content":"pub fn omega_inv(e: Either(Nil, a)) -&gt; Maybe(a)\n Inverse morphism from `Either(Nil, a)` to `Maybe(a)`. \\\n __1 + a__\nSynonyms:\ncat/algebra.omega_inv\ncat/algebra omega_inv","url":"cat/algebra.html#omega_inv"},{"doc":"cat/algebra","title":"omicron","content":"pub fn omicron(f: fn(a) -&gt; fn(b) -&gt; c) -&gt; fn(Pair(b, a)) -&gt; c\n `Exponentials of Exponentials`. \\\n __(a ^ b) ^ c = a ^ (b * c)__\nSynonyms:\ncat/algebra.omicron\ncat/algebra omicron","url":"cat/algebra.html#omicron"},{"doc":"cat/algebra","title":"omicron_inv","content":"pub fn omicron_inv(g: fn(Pair(a, b)) -&gt; c) -&gt; fn(b) -&gt; fn(a) -&gt; c\n Inverse `Exponentials of Exponentials`. \\\n __(a ^ b) ^ c = a ^ (b * c)__\nSynonyms:\ncat/algebra.omicron_inv\ncat/algebra omicron_inv","url":"cat/algebra.html#omicron_inv"},{"doc":"cat/algebra","title":"product_to_sum","content":"pub fn product_to_sum(\n  p: Pair(a, Either(b, c)),\n) -&gt; Either(Pair(a, b), Pair(a, c))\n Distributivity `up to isomorphism`. \\\n __a * (b + c) = a * b + a * c__\nSynonyms:\ncat/algebra.product_to_sum\ncat/algebra product_to_sum","url":"cat/algebra.html#product_to_sum"},{"doc":"cat/algebra","title":"psi","content":"pub fn psi(e: Either(a, Void)) -&gt; a\n Morphism from `Either(a, Void)` to `a`. \\\n __a + 0 = a__\nSynonyms:\ncat/algebra.psi\ncat/algebra psi","url":"cat/algebra.html#psi"},{"doc":"cat/algebra","title":"psi_inv","content":"pub fn psi_inv(x: a) -&gt; Either(a, Void)\n Inverse morphism from `a` to `Either(a, Void)`. \\\n __a = a + 0__\nSynonyms:\ncat/algebra.psi_inv\ncat/algebra psi_inv","url":"cat/algebra.html#psi_inv"},{"doc":"cat/algebra","title":"rho","content":"pub fn rho(p: Pair(a, Nil)) -&gt; a\n Morphism from `(a, ())` to `a`. \\\n __a * 1 = a__\nSynonyms:\ncat/algebra.rho\ncat/algebra rho","url":"cat/algebra.html#rho"},{"doc":"cat/algebra","title":"rho_inv","content":"pub fn rho_inv(x: a) -&gt; Pair(a, Nil)\n Inverse morphism from `a` to `(a, ())`. \\\n __a = a * 1__\nSynonyms:\ncat/algebra.rho_inv\ncat/algebra rho_inv","url":"cat/algebra.html#rho_inv"},{"doc":"cat/algebra","title":"sum_to_product","content":"pub fn sum_to_product(\n  e: Either(Pair(a, b), Pair(a, c)),\n) -&gt; Pair(a, Either(b, c))\n Inverse distributivity `up to isomorphism`. \\\n __a * b + a * c = a * (b + c)__\nSynonyms:\ncat/algebra.sum_to_product\ncat/algebra sum_to_product","url":"cat/algebra.html#sum_to_product"},{"doc":"cat/algebra","title":"swap","content":"pub fn swap(p: Pair(a, b)) -&gt; Pair(b, a)\n Commutativity `up to isomorphism` for Pair. \\\n __a * b = b * a__\n ### Examples\n \n swap(Pair(5, &quot;abc&quot;))\n // -&gt; Pair(&quot;abc&quot;, 5)\n \nSynonyms:\ncat/algebra.swap\ncat/algebra swap","url":"cat/algebra.html#swap"},{"doc":"cat/algebra","title":"switch","content":"pub fn switch(e: Either(a, b)) -&gt; Either(b, a)\n Commutativity `up to isomorphism` for Either. \\\n __a + b = b + a__\n ### Examples\n \n swap(Left(5))\n // -&gt; Right(5)\n swap(Right(&quot;abc&quot;))\n // -&gt; Left(&quot;abc&quot;)\n \nSynonyms:\ncat/algebra.switch\ncat/algebra switch","url":"cat/algebra.html#switch"},{"doc":"cat/algebra","title":"theta","content":"pub fn theta(select: fn(Nil) -&gt; a) -&gt; a\n `First Power`. \\\n __a ^ 1 = a__\nSynonyms:\ncat/algebra.theta\ncat/algebra theta","url":"cat/algebra.html#theta"},{"doc":"cat/algebra","title":"theta_inv","content":"pub fn theta_inv(x: a) -&gt; fn(Nil) -&gt; a\n Inverse `First Power`. \\\n __a = a ^ 1__\nSynonyms:\ncat/algebra.theta_inv\ncat/algebra theta_inv","url":"cat/algebra.html#theta_inv"},{"doc":"cat/algebra","title":"transitivity","content":"pub fn transitivity(\n  p: Pair(fn(a) -&gt; b, fn(b) -&gt; c),\n) -&gt; fn(a) -&gt; c\n `Chain Rule`. \\\n __(a -&gt; b) ∧ (b -&gt; c) -&gt; (a -&gt; c)__\nSynonyms:\ncat/algebra.transitivity\ncat/algebra transitivity","url":"cat/algebra.html#transitivity"},{"doc":"cat/algebra","title":"upsilon","content":"pub fn upsilon(\n  f: fn(a) -&gt; Pair(b, c),\n) -&gt; Pair(fn(a) -&gt; b, fn(a) -&gt; c)\n `Exponentials over Products`. \\\n __(a * b) ^ c = (a ^ c) * (b ^ c)__\nSynonyms:\ncat/algebra.upsilon\ncat/algebra upsilon","url":"cat/algebra.html#upsilon"},{"doc":"cat/algebra","title":"upsilon_inv","content":"pub fn upsilon_inv(\n  g: Pair(fn(a) -&gt; b, fn(a) -&gt; c),\n) -&gt; fn(a) -&gt; Pair(b, c)\n Inverse `Exponentials over Products`. \\\n __(a ^ c) * (b ^ c) = (a * b) ^ c__\nSynonyms:\ncat/algebra.upsilon_inv\ncat/algebra upsilon_inv","url":"cat/algebra.html#upsilon_inv"},{"doc":"cat/algebra","title":"zeta","content":"pub fn zeta(absurd: fn(Void) -&gt; a) -&gt; Nil\n `Zero-th Power`. \\\n __a ^ 0 = 1__\nSynonyms:\ncat/algebra.zeta\ncat/algebra zeta","url":"cat/algebra.html#zeta"},{"doc":"cat/algebra","title":"zeta_inv","content":"pub fn zeta_inv(unit: Nil) -&gt; fn(Void) -&gt; a\n Inverse `Zero-th Power`. \\\n __1 = a ^ 0__\nSynonyms:\ncat/algebra.zeta_inv\ncat/algebra zeta_inv","url":"cat/algebra.html#zeta_inv"},{"doc":"cat/algebra","title":"cat/algebra","content":" `Isomorphisms` (morphisms that are invertible) describing various equations. \\\n Algebra on Types (Void = 0, Nil = 1, Bool = 2, Either(a, b) = a + b, Pair(a, b) = a * b, Maybe(a) = 1 + a, Isomorphism = ==, Function fn(a) -&gt; b = Exponential b ^ a)","url":"cat/algebra.html"},{"doc":"cat/functor","title":"Functor","content":"pub type Functor(f, a, b, fa, fb) {\n  Functor(fmap: fn(fn(a) -&gt; b) -&gt; fn(fa) -&gt; fb)\n}\n `Functor` type in gleam.\n \n // Haskell type class\n class Functor f where\n     fmap :: (a -&gt; b) -&gt; f a -&gt; f b\n \n ### Functor laws\n - Preservation of `identity`: **fmap id = id**\n - Preservation of `composition`: **fmap (g ∘ h) = (fmap g) ∘ (fmap h)**\n \n Since gleam does not have `Higher Kinded Types`, we cannot pass the type constructor to the Functor type. \\\n We would like pass Option as f and then use f(a) as a type in the fmap definition.\n ### Compromise\n The user will follow this **convention**:\n - f: the first parameter of the Functor type is a `phantom type` used to differentiate between Functor instances\n - a, b: the second and third parameters are `generic types`\n - fa, fb: the fourth and fifth parameters are the `constructed types` **f(a)** and **f(b)** respectively\n ### Examples \n \n // The type that will be an instance of Functor:\n type Identity(a) {\n   Identity(a)\n }\n // The phantom type that will be the first parameter:\n type IdentityF\n // Instance of Identity for Functor (defined as a function to keep it generic over a and b)\n let id_functor = fn() -&gt; Functor(IdentityF, a, b, Identity(a), Identity(b)){\n     Functor(fmap: fn(f) {\n         fn(idx) {\n             let Identity(x) = idx\n             Identity(f(x))\n         }\n     })\n }\n // Use: the function id_functor needs to be called twice\n // Each time it binds a and b (first to Int and Bool, second to Float and String)\n let f = fn(x: Int) -&gt; Bool { x % 2 == 0 }\n identity_functor().fmap(f)(Identity(5))\n // -&gt; Identity(False)\n let g = fn(x: Float) -&gt; String { float.to_string(x) }\n identity_functor().fmap(g)(Identity(6.0))\n // -&gt; Identity(&quot;6.0&quot;)\n \nFunctor(fmap: fn(fn(a) -&gt; b) -&gt; fn(fa) -&gt; fb)\n\n\nSynonyms:\ncat/functor.Functor\ncat/functor Functor","url":"cat/functor.html#Functor"},{"doc":"cat/functor","title":"functor_compose","content":"pub fn functor_compose(\n  g: Functor(a, b, c, d, e),\n  f: Functor(f, g, h, b, c),\n) -&gt; Functor(Pair(a, f), g, h, d, e)\n Functor `composition`. \\\n Is simply the composition of their fmaps.\n ### Examples\n \n Some([1, 2, 3])\n |&gt; functor_compose(option_functor(), list_functor()).fmap\n ( fn(x) { int.to_string(x + 1) } )\n // -&gt; Some([&quot;2&quot;, &quot;3&quot;, &quot;4&quot;])\n \nSynonyms:\ncat/functor.functor_compose\ncat/functor functor_compose","url":"cat/functor.html#functor_compose"},{"doc":"cat/functor","title":"replace","content":"pub fn replace(functor: Functor(a, b, c, d, e)) -&gt; fn(c, d) -&gt; e\n Haskell `(&lt;$)` operator.\n \n (&lt;$) :: a -&gt; f b -&gt; f a\n x &lt;$ m = fmap (const x) m \n \n ### Examples\n \n replace(option_functor())(&quot;a&quot;, Some(2))\n // -&gt; Some(&quot;a&quot;)\n replace(option_functor())(&quot;a&quot;, None)\n // -&gt; None\n \nSynonyms:\ncat/functor.replace\ncat/functor replace","url":"cat/functor.html#replace"},{"doc":"cat/functor","title":"cat/functor","content":" `Functor` type {minimal implementation - `fmap`}. \\\n Default implementation for: `replace` (&lt;$ operator). \\\n Functor `composition`.","url":"cat/functor.html"},{"doc":"cat/applicative","title":"Applicative","content":"pub opaque type Applicative(f, a, fab, fa, fb)\n `Applicative` type.\n \n class Functor f =&gt; Applicative f where\n     pure :: a -&gt; f a\n     (&lt;*&gt;) :: f (a -&gt; b) -&gt; f a -&gt; f b\n \n\nSynonyms:\ncat/applicative.Applicative\ncat/applicative Applicative","url":"cat/applicative.html#Applicative"},{"doc":"cat/applicative","title":"apply","content":"pub fn apply(\n  applicative: Applicative(Functor(a, b, c, d, e), b, f, d, e),\n) -&gt; fn(f) -&gt; fn(d) -&gt; e\n Getter for Applicative `apply`.\nSynonyms:\ncat/applicative.apply\ncat/applicative apply","url":"cat/applicative.html#apply"},{"doc":"cat/applicative","title":"fmap","content":"pub fn fmap(\n  applicative: Applicative(Functor(a, b, c, d, e), b, f, d, e),\n) -&gt; fn(fn(b) -&gt; c) -&gt; fn(d) -&gt; e\n Getter for Applicative `fmap`.\nSynonyms:\ncat/applicative.fmap\ncat/applicative fmap","url":"cat/applicative.html#fmap"},{"doc":"cat/applicative","title":"new","content":"pub fn new(\n  f: fn() -&gt; Functor(a, b, c, d, e),\n  pure: fn(b) -&gt; d,\n  apply: fn(f) -&gt; fn(d) -&gt; e,\n) -&gt; Applicative(Functor(a, b, c, d, e), b, f, d, e)\n `Smart constructor` for the `Applicative` type.\nSynonyms:\ncat/applicative.new\ncat/applicative new","url":"cat/applicative.html#new"},{"doc":"cat/applicative","title":"pure","content":"pub fn pure(\n  applicative: Applicative(Functor(a, b, c, d, e), b, f, d, e),\n) -&gt; fn(b) -&gt; d\n Getter for Applicative `pure`.\nSynonyms:\ncat/applicative.pure\ncat/applicative pure","url":"cat/applicative.html#pure"},{"doc":"cat/applicative","title":"cat/applicative","content":" `Applicative` type {minimal implementations - `pure` and `apply`}.","url":"cat/applicative.html"},{"doc":"cat/bifunctor","title":"BiComp","content":"pub type BiComp(bf, fu, gu, a, b, fua, gub, bifgab) {\n  BiComp(bifgab)\n}\n Bifunctor composition type. \n \n newtype BiComp bf fu gu a b = BiComp (bf (fu a ) (gu b))\n \nBiComp(bifgab)\n\n\nSynonyms:\ncat/bifunctor.BiComp\ncat/bifunctor BiComp","url":"cat/bifunctor.html#BiComp"},{"doc":"cat/bifunctor","title":"BiCompF","content":"pub type BiCompF(bf, fu, gu)\n Phantom type for bifunctor composition.\n\nSynonyms:\ncat/bifunctor.BiCompF\ncat/bifunctor BiCompF","url":"cat/bifunctor.html#BiCompF"},{"doc":"cat/bifunctor","title":"Bifunctor","content":"pub type Bifunctor(f, a, b, c, d, fab, fcd) {\n  Bifunctor(bimap: fn(fn(a) -&gt; c, fn(b) -&gt; d) -&gt; fn(fab) -&gt; fcd)\n}\n `Bifunctor` type in gleam.\n \n // Haskell type class\n class Bifunctor f where\n     bimap :: (a -&gt; c) -&gt; (b -&gt; d) -&gt; f a b -&gt; f c d\n     first :: (a -&gt; c) -&gt; f a b -&gt; f c b\n     first g = bimap g id\n     second :: (b -&gt; d) -&gt; f a b -&gt; f a d\n     second = bimap id\n \nBifunctor(bimap: fn(fn(a) -&gt; c, fn(b) -&gt; d) -&gt; fn(fab) -&gt; fcd)\n\n\nSynonyms:\ncat/bifunctor.Bifunctor\ncat/bifunctor Bifunctor","url":"cat/bifunctor.html#Bifunctor"},{"doc":"cat/bifunctor","title":"bifunctor_compose","content":"pub fn bifunctor_compose(\n  bf_instance: Bifunctor(a, b, c, d, e, f, g),\n  fu_instance: Functor(h, i, j, b, d),\n  gu_instance: Functor(k, l, m, c, e),\n) -&gt; Bifunctor(BiCompF(a, h, k), i, l, j, m, f, g)\n Composition of a `Bifunctor` with `2 Functors`.\n \n // Haskell instance\n instance (Bifunctor bf, Functor fu, Functor gu) =&gt;\n     Bifunctor (BiComp bf fu gu) where\n         bimap f1 f2 (BiComp x) = BiComp ((bimap (fmap f1) (fmap f2)) x)\n \n ### Examples\n \n Right(Identity(3))\n |&gt; bifunctor_compose(either_bifunctor(), const_functor(), identity_functor())\n   .bimap(fn(_) { panic }, fn(x) { x % 2 == 0 })\n // -&gt; Right(Identity(False))\n \nSynonyms:\ncat/bifunctor.bifunctor_compose\ncat/bifunctor bifunctor_compose","url":"cat/bifunctor.html#bifunctor_compose"},{"doc":"cat/bifunctor","title":"first","content":"pub fn first(\n  bifunctor: Bifunctor(a, b, c, d, c, e, f),\n) -&gt; fn(fn(b) -&gt; d) -&gt; fn(e) -&gt; f\n Default function `first` for a given Bifunctor instance.\n ### Examples\n \n let first_show =\n     int.to_string\n     |&gt; {\n         pair_bifunctor()\n         |&gt; first\n     }\n\n first_show(Pair(8, 9))\n // -&gt; Pair(&quot;8&quot;, 9)\n \nSynonyms:\ncat/bifunctor.first\ncat/bifunctor first","url":"cat/bifunctor.html#first"},{"doc":"cat/bifunctor","title":"second","content":"pub fn second(\n  bifunctor: Bifunctor(a, b, c, b, d, e, f),\n) -&gt; fn(fn(c) -&gt; d) -&gt; fn(e) -&gt; f\n Default function `second` for a given Bifunctor instance.\n ### Examples\n \n let second_show = second(either_bifunctor())(int.to_string)\n\n Left(8)\n |&gt; second_show\n // -&gt; Left(8)\n Right(9)\n |&gt; second_show\n // -&gt; Right(&quot;9&quot;)\n \nSynonyms:\ncat/bifunctor.second\ncat/bifunctor second","url":"cat/bifunctor.html#second"},{"doc":"cat/bifunctor","title":"cat/bifunctor","content":" `Bifunctor` type {minimal implementation - `bimap`}. \\\n Default functions: `first` and `second` (defined in terms of bimap). \\\n Bifunctor `composition`.","url":"cat/bifunctor.html"},{"doc":"cat/contravariant","title":"Contravariant","content":"pub type Contravariant(f, a, b, fa, fb) {\n  Contravariant(contramap: fn(fn(a) -&gt; b) -&gt; fn(fb) -&gt; fa)\n}\n `Contravariant` type in gleam.\n \n // Haskell type class\n class Contravariant f where\n     contramap :: (a -&gt; b) -&gt; f b -&gt; f a\n \n ### Contravariant laws:\n - Preservation of `identity`: **contramap id = id**\n - Preservation of `composition`: **contramap (g ∘ h) = (contramap h) ∘ (contramap g)** \n \n See [`functor type`](#Functor) for convention.\nContravariant(contramap: fn(fn(a) -&gt; b) -&gt; fn(fb) -&gt; fa)\n\n\nSynonyms:\ncat/contravariant.Contravariant\ncat/contravariant Contravariant","url":"cat/contravariant.html#Contravariant"},{"doc":"cat/contravariant","title":"phantom","content":"pub fn phantom(\n  functor: Functor(a, b, Nil, c, d),\n  contra: Contravariant(e, b, Nil, f, d),\n) -&gt; fn(c) -&gt; f\n Haskell `phantom` function.\n \n phantom :: (Functor f, Contravariant f) =&gt; f a =&gt; f b\n phantom x = () &lt;$ x $&lt; ()\n \n If `f `is both `Functor` and `Contravariant`, it can&#39;t use its argument in a meaningful way. The laws follow from the preservation of `composition` for `fmap` and `contramap`.\n ### Laws:\n - __fmap f = phantom__\n - __contramap f = phantom__\n ### Examples\n \n // Phantom type for the instance\n pub type UnitF\n // Construct two instances (functor and covariant) for the same type\n let unit_functor: Functor(UnitF, _, _, _, Nil) =\n     Functor(fmap: fn(_) { cat.unit })\n let unit_contravariant: Contravariant(UnitF, _, _, Nil, _) =\n     Contravariant(contramap: fn(_) { cat.unit })\n // We are left with the instance UnitF\n // According to the laws of composition for fmap and contramap, this type can&#39;t do anything\n &quot;abc&quot;\n |&gt; phantom(unit_functor, unit_contravariant)\n // -&gt; Nil\n \nSynonyms:\ncat/contravariant.phantom\ncat/contravariant phantom","url":"cat/contravariant.html#phantom"},{"doc":"cat/contravariant","title":"replace","content":"pub fn replace(\n  contravar: Contravariant(a, b, c, d, e),\n) -&gt; fn(c, e) -&gt; d\n Haskell `(&gt;$)` operator.\n \n (&gt;$) :: b -&gt; f b -&gt; f a\n (&gt;$) = contramap ∘ const\n \n ### Examples\n \n let o = con.Op(int.to_string)\n // Doesn&#39;t matter what value/type we send to the final apply  \n True\n |&gt; replace(op_contravariant())(7, o).apply\n // -&gt; &quot;7&quot;\n \nSynonyms:\ncat/contravariant.replace\ncat/contravariant replace","url":"cat/contravariant.html#replace"},{"doc":"cat/contravariant","title":"replace_flip","content":"pub fn replace_flip(\n  contravar: Contravariant(a, b, c, d, e),\n) -&gt; fn(e, c) -&gt; d\n Haskell `($&lt;)` operator.\n \n ($&lt;) :: f b -&gt; b -&gt; f a\n ($&lt;) = flip(&gt;$)\n \n ### Examples\n \n let o = con.Op(int.to_string)\n // Doesn&#39;t matter what value/type we use for the final apply  \n [1, 2, 3]\n |&gt; replace_flip(op_contravariant())(o, 7).apply\n // -&gt; &quot;7&quot;\n \nSynonyms:\ncat/contravariant.replace_flip\ncat/contravariant replace_flip","url":"cat/contravariant.html#replace_flip"},{"doc":"cat/contravariant","title":"cat/contravariant","content":" `Contravariant` functor type {minimal implementation - `contramap`}. \\\n Default implementations for: `replace`, `replace_flip` (&gt;$ and $&lt; operators), and `phantom` function.","url":"cat/contravariant.html"},{"doc":"cat/instances/functor","title":"ConstF","content":"pub type ConstF(c)\n Phantom type for `Const Functor`. \\\n We bind the first parameter of Const.\n\nSynonyms:\ncat/instances/functor.ConstF\ncat/instances/functor ConstF","url":"cat/instances/functor.html#ConstF"},{"doc":"cat/instances/functor","title":"EitherF","content":"pub type EitherF(a)\n Phantom type for `Either Functor`.\n\nSynonyms:\ncat/instances/functor.EitherF\ncat/instances/functor EitherF","url":"cat/instances/functor.html#EitherF"},{"doc":"cat/instances/functor","title":"FunctionF","content":"pub type FunctionF(r)\n Phantom type for `Function Functor`.\n\nSynonyms:\ncat/instances/functor.FunctionF\ncat/instances/functor FunctionF","url":"cat/instances/functor.html#FunctionF"},{"doc":"cat/instances/functor","title":"IdentityF","content":"pub type IdentityF\n\n\nSynonyms:\ncat/instances/functor.IdentityF\ncat/instances/functor IdentityF","url":"cat/instances/functor.html#IdentityF"},{"doc":"cat/instances/functor","title":"ListF","content":"pub type ListF\n Phantom type for `List Functor`.\n\nSynonyms:\ncat/instances/functor.ListF\ncat/instances/functor ListF","url":"cat/instances/functor.html#ListF"},{"doc":"cat/instances/functor","title":"OptionF","content":"pub type OptionF\n Phantom type for `Option Functor`.\n\nSynonyms:\ncat/instances/functor.OptionF\ncat/instances/functor OptionF","url":"cat/instances/functor.html#OptionF"},{"doc":"cat/instances/functor","title":"PairF","content":"pub type PairF(a)\n Phantom type for `Pair Functor`.\n\nSynonyms:\ncat/instances/functor.PairF\ncat/instances/functor PairF","url":"cat/instances/functor.html#PairF"},{"doc":"cat/instances/functor","title":"ReaderF","content":"pub type ReaderF(r)\n Phantom type for `Reader Functor`.\n\nSynonyms:\ncat/instances/functor.ReaderF\ncat/instances/functor ReaderF","url":"cat/instances/functor.html#ReaderF"},{"doc":"cat/instances/functor","title":"TripleF","content":"pub type TripleF(a, b)\n Phantom type for `Triple Functor`.\n\nSynonyms:\ncat/instances/functor.TripleF\ncat/instances/functor TripleF","url":"cat/instances/functor.html#TripleF"},{"doc":"cat/instances/functor","title":"TupleF","content":"pub type TupleF(a)\n Phantom type for `Tuple Functor`.\n\nSynonyms:\ncat/instances/functor.TupleF\ncat/instances/functor TupleF","url":"cat/instances/functor.html#TupleF"},{"doc":"cat/instances/functor","title":"WriterF","content":"pub type WriterF\n Phantom type for `Writer Functor`.\n\nSynonyms:\ncat/instances/functor.WriterF\ncat/instances/functor WriterF","url":"cat/instances/functor.html#WriterF"},{"doc":"cat/instances/functor","title":"const_functor","content":"pub fn const_functor() -&gt; Functor(\n  ConstF(a),\n  b,\n  c,\n  Const(a, b),\n  Const(a, c),\n)\n `Const Functor Instance`.\n \n // Haskell instance\n instance Functor (Const c) where\n     fmap :: (a -&gt; b) -&gt; Const c a -&gt; Const c b\n     fmap _ (Const v) = Const v  \n \nSynonyms:\ncat/instances/functor.const_functor\ncat/instances/functor const_functor","url":"cat/instances/functor.html#const_functor"},{"doc":"cat/instances/functor","title":"either_functor","content":"pub fn either_functor() -&gt; Functor(\n  EitherF(a),\n  b,\n  c,\n  Either(a, b),\n  Either(a, c),\n)\n `Either Functor`.\n ### Examples\n \n either_functor().fmap(bool.negate)(Left(27))\n // -&gt; Left(27)\n either_functor().fmap(bool.negate)(Right(False))\n // -&gt; Right(True)\n \nSynonyms:\ncat/instances/functor.either_functor\ncat/instances/functor either_functor","url":"cat/instances/functor.html#either_functor"},{"doc":"cat/instances/functor","title":"function_functor","content":"pub fn function_functor() -&gt; Functor(\n  FunctionF(a),\n  b,\n  c,\n  fn(a) -&gt; b,\n  fn(a) -&gt; c,\n)\n `(-&gt;) Functor Instance`.\n \n // Haskell instance\n instance Functor ((-&gt;) r) where\n     fmap :: (a -&gt; b) -&gt; (r -&gt; a) -&gt; (r -&gt; b)\n     fmap f g = f ∘ g\n \n ### Examples\n \n let f = fn(x) { x % 2 == 0 }\n let g = bool.to_string\n \n function_functor().fmap(g)(f)(19)\n // -&gt; &quot;False&quot;\n \nSynonyms:\ncat/instances/functor.function_functor\ncat/instances/functor function_functor","url":"cat/instances/functor.html#function_functor"},{"doc":"cat/instances/functor","title":"identity_functor","content":"pub fn identity_functor() -&gt; Functor(\n  IdentityF,\n  a,\n  b,\n  Identity(a),\n  Identity(b),\n)\n `Identity Functor Instance`.\n \n // Haskell implementation\n instance Functor Maybe where\n     fmap :: (a -&gt; b) -&gt; Identity a -&gt; Identity b\n     fmap f (Identity x) = Identity(f x)\n \n ### Examples\n \n let f = fn(x: Int) -&gt; Bool { x % 2 == 0 }\n identity_functor().fmap(f)(Identity(5))\n // -&gt; Identity(False)\n \nSynonyms:\ncat/instances/functor.identity_functor\ncat/instances/functor identity_functor","url":"cat/instances/functor.html#identity_functor"},{"doc":"cat/instances/functor","title":"list_functor","content":"pub fn list_functor() -&gt; Functor(ListF, a, b, List(a), List(b))\n `List Functor Instance`.\n \n // Haskell instance\n instance Functor [] where\n     fmap :: (a -&gt; b) -&gt; [a] -&gt; [b]\n     fmap _ [] = []\n     fmap f (x:xs) = (f x):(fmap f xs)\n \nSynonyms:\ncat/instances/functor.list_functor\ncat/instances/functor list_functor","url":"cat/instances/functor.html#list_functor"},{"doc":"cat/instances/functor","title":"option_functor","content":"pub fn option_functor() -&gt; Functor(\n  OptionF,\n  a,\n  b,\n  Option(a),\n  Option(b),\n)\n `Option Functor Instance` (generic over a and b).\n \n // Haskell instance\n instance Functor Maybe where\n     fmap :: (a -&gt; b) -&gt; Maybe a -&gt; Maybe b\n     fmap _ Nothing = Nothing\n     fmap f (Just x) = Just (f x)\n \n ### Examples\n \n let double = fn(x) { x * 2 }\n option_functor().fmap(double)(None)\n // -&gt; None\n Some(2)\n |&gt; option_functor().fmap(double)\n // -&gt; Some(4)\n \nSynonyms:\ncat/instances/functor.option_functor\ncat/instances/functor option_functor","url":"cat/instances/functor.html#option_functor"},{"doc":"cat/instances/functor","title":"pair_functor","content":"pub fn pair_functor() -&gt; Functor(\n  PairF(a),\n  b,\n  c,\n  Pair(a, b),\n  Pair(a, c),\n)\n `Pair Functor`.\n ### Examples\n \n pair_functor().fmap(bool.negate)(Pair(9, False))\n // -&gt; Pair(9, True)\n \nSynonyms:\ncat/instances/functor.pair_functor\ncat/instances/functor pair_functor","url":"cat/instances/functor.html#pair_functor"},{"doc":"cat/instances/functor","title":"reader_functor","content":"pub fn reader_functor() -&gt; Functor(\n  ReaderF(a),\n  b,\n  c,\n  Reader(a, b),\n  Reader(a, c),\n)\n `Reader Functor Instance`.\n \n // Haskell implementation\n instance Functor (Reader r) where\n   fmap :: (a -&gt; b) -&gt; Reader r a -&gt; Reader r b\n   fmap f g = f ∘ g\n \n ### Examples\n \n let ra = Reader(fn(x) { x % 2 == 0 })\n let f = bool.to_string\n\n reader_functor().fmap(f)(ra).apply(19)\n // -&gt; &quot;False&quot;\n \nSynonyms:\ncat/instances/functor.reader_functor\ncat/instances/functor reader_functor","url":"cat/instances/functor.html#reader_functor"},{"doc":"cat/instances/functor","title":"triple_functor","content":"pub fn triple_functor() -&gt; Functor(\n  TripleF(a, b),\n  c,\n  d,\n  #(a, b, c),\n  #(a, b, d),\n)\n `Triple Functor`.\n ### Examples\n \n triple_functor().fmap(bool.negate)(#(&quot;abc&quot;, 9, False))\n // -&gt; #(&quot;abc&quot;, 9, True)\n \nSynonyms:\ncat/instances/functor.triple_functor\ncat/instances/functor triple_functor","url":"cat/instances/functor.html#triple_functor"},{"doc":"cat/instances/functor","title":"tuple_functor","content":"pub fn tuple_functor() -&gt; Functor(\n  TupleF(a),\n  b,\n  c,\n  #(a, b),\n  #(a, c),\n)\n `Tuple Functor`.\n ### Examples\n \n tuple_functor().fmap(bool.negate)(#(9, False))\n // -&gt; #(9, True)\n \nSynonyms:\ncat/instances/functor.tuple_functor\ncat/instances/functor tuple_functor","url":"cat/instances/functor.html#tuple_functor"},{"doc":"cat/instances/functor","title":"writer_functor","content":"pub fn writer_functor() -&gt; Functor(\n  WriterF,\n  a,\n  b,\n  Writer(a),\n  Writer(b),\n)\n `Writer Functor Instance`.\n \n // Haskell Instance\n fmap :: (a -&gt; b) -&gt; Writer a -&gt; Writer b\n fmap f = id &gt;=&gt; (\\x -&gt; return (f x))\n \n ### Examples\n \n monad.Writer(16, &quot;message&quot;)\n |&gt; writer_functor().fmap(fn(x) { x % 4 == 0 })\n // -&gt; monad.Writer(True, &quot;message&quot;)\n \nSynonyms:\ncat/instances/functor.writer_functor\ncat/instances/functor writer_functor","url":"cat/instances/functor.html#writer_functor"},{"doc":"cat/instances/functor","title":"cat/instances/functor","content":" `Functor` instances: Option, List, Reader, Writer, Const, Tuple, Triple, Pair, Either.","url":"cat/instances/functor.html"},{"doc":"cat/instances/applicative","title":"list_applicative","content":"pub fn list_applicative() -&gt; Applicative(\n  Functor(ListF, a, b, List(a), List(b)),\n  a,\n  List(fn(a) -&gt; b),\n  List(a),\n  List(b),\n)\n Instance for `Applicative List`.\n \n instance Applicative [] where\n     // pure :: a -&gt; [a]\n     pure x = [x]\n     // (&lt;*&gt;) :: [a -&gt; b] -&gt; [a] -&gt; [b]\n     fs &lt;*&gt; xs = [f x | f &lt;- fs, x &lt;- xs]\n \n ### Examples\n \n [1, 2, 3]\n |&gt; {\n     [fn(x) { x * 2 }, fn(x) { x + 10 }]\n     |&gt; apply(list_applicative())\n }\n // -&gt; [2, 4, 6, 11, 12, 13]\n \nSynonyms:\ncat/instances/applicative.list_applicative\ncat/instances/applicative list_applicative","url":"cat/instances/applicative.html#list_applicative"},{"doc":"cat/instances/applicative","title":"option_applicative","content":"pub fn option_applicative() -&gt; Applicative(\n  Functor(OptionF, a, b, Option(a), Option(b)),\n  a,\n  Option(fn(a) -&gt; b),\n  Option(a),\n  Option(b),\n)\n Instance for `Applicative Option`.\n \n instance Applicative Maybe where\n     // pure :: a -&gt; Maybe a\n     pure x = Just x\n     // (&lt;*&gt;) :: Maybe (a -&gt; b) -&gt; Maybe a -&gt; Maybe b \n     Nothing &lt;*&gt; _ = Nothing\n     Just(f) &lt;*&gt; m = fmap f m\n \n ### Examples\n \n 9\n |&gt; { option_applicative() |&gt; pure() }\n // -&gt; Some(9)\n let option_f =\n     int.to_string\n     |&gt; { option_applicative() |&gt; pure() }\n     |&gt; { option_applicative() |&gt; apply() }\n None \n |&gt; option_f()\n // -&gt; None\n Some(12) \n |&gt; option_f()\n // -&gt; Some(&quot;12&quot;)\n \nSynonyms:\ncat/instances/applicative.option_applicative\ncat/instances/applicative option_applicative","url":"cat/instances/applicative.html#option_applicative"},{"doc":"cat/instances/applicative","title":"reader_applicative","content":"pub fn reader_applicative() -&gt; Applicative(\n  Functor(ReaderF(a), b, c, Reader(a, b), Reader(a, c)),\n  b,\n  Reader(a, fn(b) -&gt; c),\n  Reader(a, b),\n  Reader(a, c),\n)\n Applicative instance for `Reader`.\nSynonyms:\ncat/instances/applicative.reader_applicative\ncat/instances/applicative reader_applicative","url":"cat/instances/applicative.html#reader_applicative"},{"doc":"cat/instances/applicative","title":"writer_applicative","content":"pub fn writer_applicative() -&gt; Applicative(\n  Functor(WriterF, a, b, Writer(a), Writer(b)),\n  a,\n  Writer(fn(a) -&gt; b),\n  Writer(a),\n  Writer(b),\n)\n Applicative instance for `Writer`.\nSynonyms:\ncat/instances/applicative.writer_applicative\ncat/instances/applicative writer_applicative","url":"cat/instances/applicative.html#writer_applicative"},{"doc":"cat/instances/applicative","title":"cat/instances/applicative","content":" `Applicative` instances: Option, List, Reader, Writer.","url":"cat/instances/applicative.html"},{"doc":"cat/instances/bifunctor","title":"EitherBF","content":"pub type EitherBF\n Phantom type for `Either Bifunctor`.\n\nSynonyms:\ncat/instances/bifunctor.EitherBF\ncat/instances/bifunctor EitherBF","url":"cat/instances/bifunctor.html#EitherBF"},{"doc":"cat/instances/bifunctor","title":"PairBF","content":"pub type PairBF\n Phantom type for `Pair Bifunctor`.\n\nSynonyms:\ncat/instances/bifunctor.PairBF\ncat/instances/bifunctor PairBF","url":"cat/instances/bifunctor.html#PairBF"},{"doc":"cat/instances/bifunctor","title":"TupleBF","content":"pub type TupleBF\n Phantom type for `Tuple Bifunctor`.\n\nSynonyms:\ncat/instances/bifunctor.TupleBF\ncat/instances/bifunctor TupleBF","url":"cat/instances/bifunctor.html#TupleBF"},{"doc":"cat/instances/bifunctor","title":"either_bifunctor","content":"pub fn either_bifunctor() -&gt; Bifunctor(\n  EitherBF,\n  a,\n  b,\n  c,\n  d,\n  Either(a, b),\n  Either(c, d),\n)\n `Either Bifunctor`.\n ### Examples\n \n let show_or_double = either_bifunctor().bimap(int.to_string, fn(x) { x * 2 })\n \n Left(10)\n |&gt; show_or_double()\n // -&gt; should.equal(cat.Left(&quot;10&quot;))\n Right(10)\n |&gt; show_or_double()\n // -&gt; Right(20)\n \nSynonyms:\ncat/instances/bifunctor.either_bifunctor\ncat/instances/bifunctor either_bifunctor","url":"cat/instances/bifunctor.html#either_bifunctor"},{"doc":"cat/instances/bifunctor","title":"pair_bifunctor","content":"pub fn pair_bifunctor() -&gt; Bifunctor(\n  PairBF,\n  a,\n  b,\n  c,\n  d,\n  Pair(a, b),\n  Pair(c, d),\n)\n `Pair Bifunctor`.\n ### Examples\n \n Pair(2, 3)\n |&gt; pair_bifunctor().bimap(fn(x) { x % 3 }, int.to_string)()\n // -&gt; Pair(2, &quot;3&quot;)\n \nSynonyms:\ncat/instances/bifunctor.pair_bifunctor\ncat/instances/bifunctor pair_bifunctor","url":"cat/instances/bifunctor.html#pair_bifunctor"},{"doc":"cat/instances/bifunctor","title":"tuple_bifunctor","content":"pub fn tuple_bifunctor() -&gt; Bifunctor(\n  TupleBF,\n  a,\n  b,\n  c,\n  d,\n  #(a, b),\n  #(c, d),\n)\n `Tuple Bifunctor`.\n ### Examples\n \n #(6, False)\n |&gt; tuple_bifunctor().bimap(fn(x) { [x] }, fn(b) { bool.to_string(b) })()\n // -&gt; #([6], &quot;False&quot;)\n \nSynonyms:\ncat/instances/bifunctor.tuple_bifunctor\ncat/instances/bifunctor tuple_bifunctor","url":"cat/instances/bifunctor.html#tuple_bifunctor"},{"doc":"cat/instances/bifunctor","title":"cat/instances/bifunctor","content":" `Bifunctor` instances: Tuple, Pair, and Either.","url":"cat/instances/bifunctor.html"},{"doc":"cat/instances/contravariant","title":"Op","content":"pub type Op(r, a) {\n  Op(apply: fn(a) -&gt; r)\n}\n Type for `reverse` functions.\n \n type Op r a = a -&gt; r\n \n ### Examples\n \n let o = Op(fn(x) { x % 2 == 1 })\n o.apply(6)\n // -&gt; False\n \nOp(apply: fn(a) -&gt; r)\n\n\nSynonyms:\ncat/instances/contravariant.Op\ncat/instances/contravariant Op","url":"cat/instances/contravariant.html#Op"},{"doc":"cat/instances/contravariant","title":"OpC","content":"pub type OpC(r)\n Phantom type for `Op Contravariant`.\n\nSynonyms:\ncat/instances/contravariant.OpC\ncat/instances/contravariant OpC","url":"cat/instances/contravariant.html#OpC"},{"doc":"cat/instances/contravariant","title":"op_contravariant","content":"pub fn op_contravariant() -&gt; Contravariant(\n  OpC(a),\n  b,\n  c,\n  Op(a, b),\n  Op(a, c),\n)\n `Op Contravariant Instance`.\n \n // Haskell implementation\n instance Contravariant (Op r) where\n     contramap :: (b -&gt; a) -&gt; Op r a -&gt; Op r b\n     contramap f g = g∘. f\n \n ### Examples\n \n let f = fn(b) {\n     case b {\n       True -&gt; 2\n       False -&gt; 4\n     }\n  }\n let original = Op(fn(x) { int.to_string(x * 2) })\n let result = op_contravariant().contramap(f)(original)\n result.apply(False)\n // -&gt; &quot;8&quot;\n \nSynonyms:\ncat/instances/contravariant.op_contravariant\ncat/instances/contravariant op_contravariant","url":"cat/instances/contravariant.html#op_contravariant"},{"doc":"cat/instances/contravariant","title":"cat/instances/contravariant","content":" `Op type` and its Contravariant instance.","url":"cat/instances/contravariant.html"},{"doc":"cat/monad","title":"Monad","content":"pub opaque type Monad(ap, a, ma, mb)\n `Monad` type.\n \n class Applicative f =&gt; Monad f where\n   (&gt;&gt;=) :: m a -&gt; (a -&gt; m b) -&gt; m b\n \n\nSynonyms:\ncat/monad.Monad\ncat/monad Monad","url":"cat/monad.html#Monad"},{"doc":"cat/monad","title":"apply","content":"pub fn apply(\n  m: Monad(\n    Applicative(Functor(a, b, c, d, e), b, f, d, e),\n    b,\n    g,\n    h,\n  ),\n) -&gt; fn(f) -&gt; fn(d) -&gt; e\n Getter for Monad `apply`.\nSynonyms:\ncat/monad.apply\ncat/monad apply","url":"cat/monad.html#apply"},{"doc":"cat/monad","title":"bind","content":"pub fn bind(\n  m: Monad(\n    Applicative(Functor(a, b, c, d, e), b, f, d, e),\n    b,\n    g,\n    h,\n  ),\n) -&gt; fn(g) -&gt; fn(fn(b) -&gt; h) -&gt; h\n Getter for Monad `bind`.\nSynonyms:\ncat/monad.bind\ncat/monad bind","url":"cat/monad.html#bind"},{"doc":"cat/monad","title":"fmap","content":"pub fn fmap(\n  m: Monad(\n    Applicative(Functor(a, b, c, d, e), b, f, d, e),\n    b,\n    g,\n    h,\n  ),\n) -&gt; fn(fn(b) -&gt; c) -&gt; fn(d) -&gt; e\n Getter for Monad `fmap`.\nSynonyms:\ncat/monad.fmap\ncat/monad fmap","url":"cat/monad.html#fmap"},{"doc":"cat/monad","title":"new","content":"pub fn new(\n  ap: fn() -&gt; Applicative(Functor(a, b, c, d, e), b, f, d, e),\n  return: fn(b) -&gt; g,\n  bind: fn(g) -&gt; fn(fn(b) -&gt; h) -&gt; h,\n) -&gt; Monad(\n  Applicative(Functor(a, b, c, d, e), b, f, d, e),\n  b,\n  g,\n  h,\n)\n Smart constructor for `Monad` type.\nSynonyms:\ncat/monad.new\ncat/monad new","url":"cat/monad.html#new"},{"doc":"cat/monad","title":"pure","content":"pub fn pure(\n  m: Monad(\n    Applicative(Functor(a, b, c, d, e), b, f, d, e),\n    b,\n    g,\n    h,\n  ),\n) -&gt; fn(b) -&gt; d\n Getter for Monad `pure`.\nSynonyms:\ncat/monad.pure\ncat/monad pure","url":"cat/monad.html#pure"},{"doc":"cat/monad","title":"return","content":"pub fn return(\n  m: Monad(\n    Applicative(Functor(a, b, c, d, e), b, f, d, e),\n    b,\n    g,\n    h,\n  ),\n) -&gt; fn(b) -&gt; g\n Getter for Monad `return`.\nSynonyms:\ncat/monad.return\ncat/monad return","url":"cat/monad.html#return"},{"doc":"cat/monad","title":"cat/monad","content":" `Monad` type {minimal implementation - `bind`}.","url":"cat/monad.html"},{"doc":"cat/instances/monad","title":"reader_monad","content":"pub fn reader_monad() -&gt; Monad(\n  Applicative(\n    Functor(ReaderF(a), b, c, Reader(a, b), Reader(a, c)),\n    b,\n    Reader(a, fn(b) -&gt; c),\n    Reader(a, b),\n    Reader(a, c),\n  ),\n  b,\n  Reader(d, b),\n  Reader(d, e),\n)\n Monad instance for `Reader`.\n \n instance Monad ((-&gt;) r) where\n   f &gt;&gt;= k = \\ r -&gt; k (f r) r\n\nSynonyms:\ncat/instances/monad.reader_monad\ncat/instances/monad reader_monad","url":"cat/instances/monad.html#reader_monad"},{"doc":"cat/instances/monad","title":"writer_monad","content":"pub fn writer_monad() -&gt; Monad(\n  Applicative(\n    Functor(WriterF, a, b, Writer(a), Writer(b)),\n    a,\n    Writer(fn(a) -&gt; b),\n    Writer(a),\n    Writer(b),\n  ),\n  a,\n  Writer(a),\n  Writer(c),\n)\n Monad instance for `Writer`.\n \n instance Monad Writer where\n  ma &gt;&gt;= k = \n   let (va, log1) = runWriter ma\n       (vb, log2) = runWriter (k va)\n   in  Writer (vb, log1 ++ log2)\n \nSynonyms:\ncat/instances/monad.writer_monad\ncat/instances/monad writer_monad","url":"cat/instances/monad.html#writer_monad"},{"doc":"cat/instances/monad","title":"cat/instances/monad","content":" Monad instances: Writer, Reader.","url":"cat/instances/monad.html"},{"doc":"cat/monoid","title":"Monoid","content":"pub type Monoid(m) {\n  Monoid(mempty: m, mappend: fn(m, m) -&gt; m)\n}\n A monoid is a `set` with a `binary operation` or a a `single object category` with a `set of morphisms` that follow the rules of composition.\n \n class Monoid m where\n   mempty  :: m\n   mappend :: m -&gt; m -&gt; m\n \n Laws:\n - mappend mempty x = x\n - mappend x mempty = x\n - mappend x (mappend y z) = mappend (mappend x y) z\n - mconcat = foldr mappend mempty\n ### Examples\n \n let int_sum_monoid = Monoid(mempty: 0, mappend: fn(x: Int, y: Int) { x + y })\n int_sum_monoid\n |&gt; mconcat([2, 3, int_sum_monoid.mempty, 4])\n |&gt; int_sum_monoid.mappend(10)\n // -&gt; 19\n let bool_and_monoid = Monoid(mempty: True, mappend: bool.and)\n True\n |&gt; bool_and_monoid.mappend(False)\n |&gt; bool_and_monoid.mappend(bool_and_monoid.mempty)\n // -&gt; False\n \nMonoid(mempty: m, mappend: fn(m, m) -&gt; m)\n\n\nSynonyms:\ncat/monoid.Monoid\ncat/monoid Monoid","url":"cat/monoid.html#Monoid"},{"doc":"cat/monoid","title":"mconcat","content":"pub fn mconcat(mono: Monoid(a), monoid_list: List(a)) -&gt; a\n Fold a `list of monoids` using mappend. \nSynonyms:\ncat/monoid.mconcat\ncat/monoid mconcat","url":"cat/monoid.html#mconcat"},{"doc":"cat/monoid","title":"cat/monoid","content":" `Monoid` type {minimal implementation - `mempty` and `mappend`}. \\\n Default implementation for the `mconcat` function (defined in terms of mempty and mappend).","url":"cat/monoid.html"},{"doc":"cat/instances/monoid","title":"all_monoid","content":"pub fn all_monoid() -&gt; Monoid(Bool)\n Monoid instance for Bool with (&amp;&amp;).\n ### Examples\n \n let bool_and_monoid = all_monoid()\n True\n |&gt; bool_and_monoid.mappend(False)\n |&gt; bool_and_monoid.mappend(bool_and_monoid.mempty)\n // -&gt; False\n \nSynonyms:\ncat/instances/monoid.all_monoid\ncat/instances/monoid all_monoid","url":"cat/instances/monoid.html#all_monoid"},{"doc":"cat/instances/monoid","title":"any_monoid","content":"pub fn any_monoid() -&gt; Monoid(Bool)\n Monoid instance for Bool with (||).\n ### Examples\n \n let bool_or_monoid = any_monoid()\n False\n |&gt; bool_or_monoid.mappend(False)\n |&gt; bool_or_monoid.mappend(bool_or_monoid.mempty)\n // -&gt; False\n \nSynonyms:\ncat/instances/monoid.any_monoid\ncat/instances/monoid any_monoid","url":"cat/instances/monoid.html#any_monoid"},{"doc":"cat/instances/monoid","title":"function_monoid","content":"pub fn function_monoid(mono_b: Monoid(a)) -&gt; Monoid(fn(b) -&gt; a)\n Monoid instance for functions where the `result type` must be a Monoid instance.\nSynonyms:\ncat/instances/monoid.function_monoid\ncat/instances/monoid function_monoid","url":"cat/instances/monoid.html#function_monoid"},{"doc":"cat/instances/monoid","title":"list_monoid","content":"pub fn list_monoid() -&gt; Monoid(List(a))\n Returns the `canonical implementation` of the `monoid type for List`.\n ### Examples\n \n let mono_list = list_monoid()\n [1, 2]\n |&gt; mono_list.mappend([3, 4, 5])\n |&gt; mono_list.mappend(mono_list.mempty)\n |&gt; mono_list.mappend([6])\n // -&gt; [1, 2, 3, 4, 5, 6]\n \nSynonyms:\ncat/instances/monoid.list_monoid\ncat/instances/monoid list_monoid","url":"cat/instances/monoid.html#list_monoid"},{"doc":"cat/instances/monoid","title":"option_monoid","content":"pub fn option_monoid(mono_a: Monoid(a)) -&gt; Monoid(Option(a))\n Returns the `canonical implementation` of the `monoid type for Option(a)`. \\\n We must have a Monoid(a) type instance.\n ### Examples\n \n let mono_string = Monoid(mempty: &quot;&quot;, mappend: fn(x: String, y: String) -&gt; String { x &lt;&gt; y })\n let mono_maybe = option_monoid(mono_string)\n \n Some(&quot;ab&quot;)\n |&gt; mono_maybe.mappend(Some(&quot;cd&quot;))\n // -&gt; Some(&quot;abcd&quot;)\n mono_maybe.mappend(Some(&quot;abc&quot;), maybe.mempty)\n // -&gt; None\n \nSynonyms:\ncat/instances/monoid.option_monoid\ncat/instances/monoid option_monoid","url":"cat/instances/monoid.html#option_monoid"},{"doc":"cat/instances/monoid","title":"triple_monoid","content":"pub fn triple_monoid(\n  mono_a: Monoid(a),\n  mono_b: Monoid(b),\n  mono_c: Monoid(c),\n) -&gt; Monoid(#(a, b, c))\n Returns the `canonical implementation` of the `monoid type for Triple`. \\\n We must have a Monoid type instance for a, b, and c.\nSynonyms:\ncat/instances/monoid.triple_monoid\ncat/instances/monoid triple_monoid","url":"cat/instances/monoid.html#triple_monoid"},{"doc":"cat/instances/monoid","title":"tuple_monoid","content":"pub fn tuple_monoid(\n  mono_a: Monoid(a),\n  mono_b: Monoid(b),\n) -&gt; Monoid(#(a, b))\n Returns the `canonical implementation` of the `monoid type for Tuple`. \\\n We must have a Monoid(a) and a Monoid(b) type instance.\nSynonyms:\ncat/instances/monoid.tuple_monoid\ncat/instances/monoid tuple_monoid","url":"cat/instances/monoid.html#tuple_monoid"},{"doc":"cat/instances/monoid","title":"unit_monoid","content":"pub fn unit_monoid() -&gt; Monoid(Nil)\n Returns the `canonical implementation` of the `monoid type for Nil` (unit type).\n ### Examples\n \n let mono_unit = unit_monoid()\n mono_unit.mappend(mono_unit.mempty, Nil)\n // -&gt; Nil\n \nSynonyms:\ncat/instances/monoid.unit_monoid\ncat/instances/monoid unit_monoid","url":"cat/instances/monoid.html#unit_monoid"},{"doc":"cat/instances/monoid","title":"cat/instances/monoid","content":" `Monoid` instances: Unit, Bool (All and Any), List, Option, Tuple, Triple, Function.","url":"cat/instances/monoid.html"},{"doc":"cat/natural","title":"NaturalTransformation","content":"pub opaque type NaturalTransformation(f, g, fa, ga)\n A natural transformation of `two functors` **F** and **G** is a collection of functions such that for every type **a** we have a function (`component`) that goes from **F a** to **G a**.\\\n This implementation in gleam is a bit more restrictive in that each `component` is an instantiation of the `generic` function **transform** for the type **a**.\n ### Examples\n A natural transformation from `Option` to `List` will contain a `transform` function that, for any type `a`, takes an `Option(a)` and returns a `List(a)`. \n \n transform: fn(Option(a)) -&gt; List(a)\n \n\nSynonyms:\ncat/natural.NaturalTransformation\ncat/natural NaturalTransformation","url":"cat/natural.html#NaturalTransformation"},{"doc":"cat/natural","title":"ffmap","content":"pub fn ffmap(\n  alpha: NaturalTransformation(Functor(a, b, c, d, e), f, g, h),\n) -&gt; fn(fn(b) -&gt; c) -&gt; fn(d) -&gt; e\n Getter for the `fmap` of `f` of `NaturalTransformation`.\nSynonyms:\ncat/natural.ffmap\ncat/natural ffmap","url":"cat/natural.html#ffmap"},{"doc":"cat/natural","title":"gfmap","content":"pub fn gfmap(\n  alpha: NaturalTransformation(a, Functor(b, c, d, e, f), g, h),\n) -&gt; fn(fn(c) -&gt; d) -&gt; fn(e) -&gt; f\n Getter for the `fmap` of `g` of `NaturalTransformation`.\nSynonyms:\ncat/natural.gfmap\ncat/natural gfmap","url":"cat/natural.html#gfmap"},{"doc":"cat/natural","title":"horizontal_composition","content":"pub fn horizontal_composition(\n  alpha: NaturalTransformation(\n    Functor(a, b, c, d, e),\n    Functor(f, b, c, g, h),\n    d,\n    g,\n  ),\n  beta: NaturalTransformation(\n    Functor(i, j, k, l, m),\n    Functor(n, j, k, o, p),\n    l,\n    o,\n  ),\n) -&gt; NaturalTransformation(\n  Functor(Pair(a, i), b, c, q, r),\n  Functor(Pair(f, n), b, c, s, t),\n  q,\n  t,\n)\n `Horizontal composition` ∘ of two `natural transformations`.\n \n alpha_a :: F a -&gt; F&#39; a\n beta_a :: G a -&gt; G&#39; a\n // These transformation compose:\n beta ∘ alpha :: G ∘ F -&gt; G&#39; ∘ F&#39;\n (beta ∘ alpha)_a = G&#39; (alpha_a) ∘ beta_Fa = beta_F&#39;a ∘ G (alpha_a)\n \n Due to gleam&#39;s type system, we cannot implement this generic function\nSynonyms:\ncat/natural.horizontal_composition\ncat/natural horizontal_composition","url":"cat/natural.html#horizontal_composition"},{"doc":"cat/natural","title":"new","content":"pub fn new(\n  f: fn() -&gt; Functor(a, b, c, d, e),\n  g: fn() -&gt; Functor(f, g, h, i, j),\n  transform: fn(d) -&gt; k,\n) -&gt; NaturalTransformation(\n  Functor(a, b, c, d, e),\n  Functor(f, g, h, i, j),\n  d,\n  k,\n)\n `Smart constructor` for the `NaturalTransformation` type. \\\n Takes two functor instance generation (so that the types stay generic) and a transform function.\nSynonyms:\ncat/natural.new\ncat/natural new","url":"cat/natural.html#new"},{"doc":"cat/natural","title":"transform","content":"pub fn transform(\n  alpha: NaturalTransformation(a, b, c, d),\n) -&gt; fn(c) -&gt; d\n Getter for the `transform` field of `NaturalTransformation`.\nSynonyms:\ncat/natural.transform\ncat/natural transform","url":"cat/natural.html#transform"},{"doc":"cat/natural","title":"vertical_composition","content":"pub fn vertical_composition(\n  alpha: NaturalTransformation(a, b, c, d),\n  beta: NaturalTransformation(b, e, d, f),\n) -&gt; NaturalTransformation(a, e, c, f)\n `Vertical composition` ⋅ of two `natural transformations`.\n \n alpha_a :: F a -&gt; G a\n beta_a :: G a -&gt; H a\n // These morphisms compose:\n beta_a ∘ alpha_a :: F a -&gt; H a\n // So the natural transformations also compose:\n (beta ⋅ alpha)_a = beta_a ∘ alpha_a\n \n ### Examples\n \n let maybe_const = \n   vertical_composition(\n     option_list_transformation(),\n     list_length_transformation()\n   )\n None\n |&gt; transform(maybe_const)\n // -&gt; Const(0)\n Some(&quot;abc&quot;)\n |&gt; transform(maybe_const)\n // -&gt; Const(1)\n \nSynonyms:\ncat/natural.vertical_composition\ncat/natural vertical_composition","url":"cat/natural.html#vertical_composition"},{"doc":"cat/natural","title":"cat/natural","content":" `NaturalTransformation` type {minimal implementation `transform`}. \\\n `Composition` function.","url":"cat/natural.html"},{"doc":"cat/instances/natural","title":"list_length_transformation","content":"pub fn list_length_transformation() -&gt; NaturalTransformation(\n  Functor(ListF, a, b, List(a), List(b)),\n  Functor(ConstF(c), d, e, Const(c, d), Const(c, e)),\n  List(a),\n  Const(Int, f),\n)\n Natural transformation from `List` to `Const Int`.\n ### Examples\n \n []\n |&gt; {list_length_transformation() |&gt; transform()}\n // -&gt; Const(0)\n [1, 2, 3, 4]\n |&gt; {list_length_transformation() |&gt; transform()}\n // -&gt; Const(4)\n \nSynonyms:\ncat/instances/natural.list_length_transformation\ncat/instances/natural list_length_transformation","url":"cat/instances/natural.html#list_length_transformation"},{"doc":"cat/instances/natural","title":"list_option_head_transformation","content":"pub fn list_option_head_transformation() -&gt; NaturalTransformation(\n  Functor(ListF, a, b, List(a), List(b)),\n  Functor(OptionF, c, d, Option(c), Option(d)),\n  List(a),\n  Option(a),\n)\n Natural transformation from `List` to `Option`.\n ### Examples\n \n []\n |&gt; {list_option_head_transformation() |&gt; transform()}\n // -&gt; None\n [1, 2, 3]\n |&gt; {list_option_head_transformation() |&gt; transform()}\n // -&gt; Some(1)\n \nSynonyms:\ncat/instances/natural.list_option_head_transformation\ncat/instances/natural list_option_head_transformation","url":"cat/instances/natural.html#list_option_head_transformation"},{"doc":"cat/instances/natural","title":"option_list_transformation","content":"pub fn option_list_transformation() -&gt; NaturalTransformation(\n  Functor(OptionF, a, b, Option(a), Option(b)),\n  Functor(ListF, c, d, List(c), List(d)),\n  Option(a),\n  List(a),\n)\n Natural transformation from `Option` to `List`.\n ### Examples\n \n None\n |&gt; {option_list_transformation() |&gt; transform()}\n // -&gt; []\n Some(7)\n |&gt; {option_list_transformation() |&gt; transform()}\n // -&gt; [7]\n \nSynonyms:\ncat/instances/natural.option_list_transformation\ncat/instances/natural option_list_transformation","url":"cat/instances/natural.html#option_list_transformation"},{"doc":"cat/instances/natural","title":"cat/instances/natural","content":" Examples of `transformations` between `functors`.","url":"cat/instances/natural.html"},{"doc":"cat/profunctor","title":"Profunctor","content":"pub type Profunctor(p, a, b, c, d, pbc, pad) {\n  Profunctor(dimap: fn(fn(a) -&gt; b, fn(c) -&gt; d) -&gt; fn(pbc) -&gt; pad)\n}\n `Profunctor`. \\\n A type constructor `p` of two arguments, which is `contra-functorial` in the `first` argument and `functorial` in the `second`.\n \n // Haskel definition\n class Profunctor p where\n   dimap :: (a -&gt; b) -&gt; (c -&gt; d) -&gt; p b c -&gt; p a d\n   dimap f g = lmap f ∘ rmap g\n   lmap :: (a -&gt; b) -&gt; p b c -&gt; p a c\n   lmap f = dimap f id\n   rmap :: (c -&gt; d) -&gt; p a c -&gt; p a d\n   rmap = dimap id\n \nProfunctor(dimap: fn(fn(a) -&gt; b, fn(c) -&gt; d) -&gt; fn(pbc) -&gt; pad)\n\n\nSynonyms:\ncat/profunctor.Profunctor\ncat/profunctor Profunctor","url":"cat/profunctor.html#Profunctor"},{"doc":"cat/profunctor","title":"lmap","content":"pub fn lmap(\n  profunctor: Profunctor(a, b, c, d, d, e, f),\n) -&gt; fn(fn(b) -&gt; c) -&gt; fn(e) -&gt; f\n Default function `lmap` for a given Profunctor instance.\n ### Examples\n \n // Function a -&gt; b ([Int] -&gt; Int)\n let f = list.fold(_, 0, fn(x, y) { x + y })\n // Profunctor pbc: Function b -&gt; c (Int -&gt; Bool)\n let h = fn(x) { x % 2 == 0 }\n // Profunctor pac: Function a -&gt; c ([Int] -&gt; Bool)\n let z = lmap(function_profunctor())(f)(h)\n [1, 2, 3]\n |&gt; z\n // -&gt; True\n [1, 2]\n |&gt; z\n // -&gt; False\n \nSynonyms:\ncat/profunctor.lmap\ncat/profunctor lmap","url":"cat/profunctor.html#lmap"},{"doc":"cat/profunctor","title":"rmap","content":"pub fn rmap(\n  profunctor: Profunctor(a, b, b, c, d, e, f),\n) -&gt; fn(fn(c) -&gt; d) -&gt; fn(e) -&gt; f\n Default function `rmap` for a given Profunctor instance.\n ### Examples\n \n // Function c -&gt; d (Bool -&gt; String)\n let g = bool.to_string\n // Profunctor pac: Function a -&gt; c ([Int] -&gt; Bool)\n let h = fn(x) { list.length(x) % 2 == 0 }\n // Profunctor pad: Function ([Int] -&gt; String)\n let z = rmap(function_profunctor())(g)(h)\n [1, 2, 3]\n |&gt; z\n // -&gt; &quot;False&quot;\n [1, 2]\n |&gt; z\n // -&gt; &quot;True&quot;\n \nSynonyms:\ncat/profunctor.rmap\ncat/profunctor rmap","url":"cat/profunctor.html#rmap"},{"doc":"cat/profunctor","title":"cat/profunctor","content":" `Profunctor` type {minimal implementation - `dimap`} \\\n Default implementations: `lmap` and `rmap`.","url":"cat/profunctor.html"},{"doc":"cat/instances/profunctor","title":"ArrowPro","content":"pub type ArrowPro\n Phantom type for the `function profunctor`.\n\nSynonyms:\ncat/instances/profunctor.ArrowPro\ncat/instances/profunctor ArrowPro","url":"cat/instances/profunctor.html#ArrowPro"},{"doc":"cat/instances/profunctor","title":"function_profunctor","content":"pub fn function_profunctor() -&gt; Profunctor(\n  ArrowPro,\n  a,\n  b,\n  c,\n  d,\n  fn(b) -&gt; c,\n  fn(a) -&gt; d,\n)\n `Profunctor` instance for the `function type` (-&gt;).\n \n // Haskell implementation\n instance Profunctor (-&gt;) where\n   dimap ab cd bc = cd ∘ bc ∘ ab\n   lmap = flip (∘)\n   rmap = (∘)\n \n ### Examples\n \n // Function a -&gt; b ([Int] -&gt; Int)\n let f = list.fold(_, 0, fn(x, y) { x + y })\n // Function c -&gt; d (Bool -&gt; String)\n let g = bool.to_string\n // Profunctor pbc: Function b -&gt; c (Int -&gt; Bool)\n let h = fn(x) { x % 2 == 0 }\n // Profunctor pad: Function a -&gt; d ([Int] -&gt; String)\n let z = function_profunctor().dimap(f, g)(h)\n [1, 2, 3]\n |&gt; z\n // -&gt; &quot;True&quot;\n [1, 2]\n |&gt; z\n // -&gt; &quot;False&quot;\n \nSynonyms:\ncat/instances/profunctor.function_profunctor\ncat/instances/profunctor function_profunctor","url":"cat/instances/profunctor.html#function_profunctor"},{"doc":"cat/instances/profunctor","title":"cat/instances/profunctor","content":" Profunctor instance for `function type`.","url":"cat/instances/profunctor.html"}]);